
;+

pro progress,percentin,bigstep=bigstepin,smallstep=smallstepin, $
                 reset=reset,last=last,label=label,noeta=noeta, $
                 frequency=frequency

;NAME:
;     PROGRESS
;PURPOSE:
;     Prints a progress summary in percent done and time remaining as
;     a process runs.  Call this routine multiple times from the
;     running process while updating the percent done parameter.
;CATEGORY:
;CALLING SEQUENCE:
;     progress,percent
;INPUTS:
;     percent = percent finished (in range 0 to 100)
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;     /reset = this must be set for the first call to set up variables.
;     /last = the process is done and this is the last call.  Optional
;             for the last call.  Makes sure the printout goes all the
;             way to 100%, does a final carriage return, and resets
;             some variables.
;     label = string to print at the front of the progress line.  Only
;             used on the first call when /reset is set.
;     bigstep = percentage multiple to print the percent done as a
;               number (def = 25).  Only used on the first call when
;               /reset is set.
;     smallstep = percentage multiple to print a dot (def = 5).  Only
;                 used on the first call when /reset is set.
;     /noeta = Do not print the estimated time to completion.  This
;              feature depends on your terminal accepting 8b as a
;              backspace character. If this does not work, the
;              formatting will be messed up.  So, if your formatting
;              is messed up, set this keyword to turn the feature off.
;     frequency = If set, update the estimated time to completion
;                 if at least 'frequency' seconds have passed since
;                 the last update.  The time is always printed when a
;                 dot or number is printed, as well. If set to 2, for
;                 example, update approximately every two seconds, etc. 
;                 If set to 0, update on every call to progress.
;                 The default (not set) is to update the time only
;                 when a dot or number is printed. 
;OUTPUTS:
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;     Assumes that nothing else is printed between calls.  If it is,
;     the formatting will be messed up.
;PROCEDURE:
;     Call the routine multiple times as progress is made.  It will
;     print numbers and dots to indicate the progress, e.g.
;     Label:  0 .... 25 .... 50 .... 75 .... 100  | Time=00:00:00
;     Also prints an estimated time to completion, HH:MM:SS
;EXAMPLE:
;     progress,0.0,/reset,label='Progress (%)'
;     for i=0,n-1 do begin
;        ; your processing goes here
;        progress,100.*float(i+1.0)/n
;     endfor
;     progress,100.,/last
;MODIFICATION HISTORY:
;     T. Metcalf 2005-Jan-06
;     2005-Jan-10 Added frequency keyword.
;-

common progress_private1,bigstep,smallstep,lastpercent,starttime,lasttime

percent = (float(percentin) < 100.0) > 0.0

if n_elements(frequency) GE 1 then freq = float(frequency[0])>0.0 else freq = 0.0

if keyword_set(reset) OR not keyword_set(bigstep) or not keyword_set(smallstep) or $
   n_elements(lastpercent) LE 0 then $
   doreset = 1 else doreset = 0
if n_elements(lastpercent) GT 0 and not keyword_set(doreset) then $
   if lastpercent LT 0.0 then doreset = 1

if keyword_set(doreset) then begin
   lastpercent = -1.0   ; must be exactly -1 at the start
   starttime = systime(1)
   lasttime = starttime
   if not keyword_set(bigstepin) then bigstep=25 else bigstep = round(bigstepin)
   if not keyword_set(smallstepin) then smallstep=5 else smallstep = round(smallstepin)
   if smallstep GT bigstep then smallstep=bigstep
   if keyword_set(label) then print,strcompress(label+': '),format='(a,$)'
endif

if percent LT lastpercent then return 

if keyword_set(last) then npercent = 100 else npercent = round(percent)
nlastpercent = round(lastpercent)

printed_something = 0
if npercent GT nlastpercent then begin
   for ipct=nlastpercent+1,npercent do begin
      if ipct MOD bigstep EQ 0 then begin
         print,' '+strcompress(string(ipct),/remove_all)+' ',format='(a,$)' 
         printed_something = 1
      endif else if ipct MOD smallstep EQ 0 then begin
         print,'.',format='(a,$)'
         printed_something = 1
      endif
   endfor
endif

if (keyword_set(printed_something) OR $
    n_elements(frequency) GE 1 OR $
    keyword_set(last)) AND $
   NOT keyword_set(doreset) AND $
   NOT keyword_set(noeta) then begin
   ; Compute & print estimated time to completion
   thistime = systime(1)
   if (thistime - lasttime) GE freq OR $
      keyword_set(printed_something) OR $
      keyword_set(last) then begin
      eta = (100.0-percent)*(thistime-starttime)/percent
      h = fix(eta / 3600.0)
      eta = eta - h*3600.0
      m = fix(eta/60.0)
      eta = eta - m*60.0
      s = fix(eta)
      if h LT 10 then h = '0'+string(h) else h = string(h)
      if m LT 10 then m = '0'+string(m) else m = string(m)
      if s LT 10 then s = '0'+string(s) else s = string(s)
      eta = ' | Time='+strcompress(h+':'+m+':'+s,/remove_all)
      ; 8b is a backspace so the eta will be overwritten
      eta = eta+string(replicate(8b,strlen(eta)))
      print,eta,format='(a,$)'
      lasttime = thistime  ; the last time the ETA was printed
   endif
endif

if keyword_set(last) then begin
   print
   lastpercent = -1.0  ; force a reset on the next call
endif

lastpercent = percent

end
